@@ -30,6 +30,8 @@ using System.Collections;
 using System.IO;
 using System.Text;
 
+using Beagle.Daemon;
+
 using HtmlAgilityPack;
 
 namespace Beagle.Filters {
@@ -38,11 +40,11 @@ namespace Beagle.Filters {
 
 		public FilterHtml ()
 		{
-			AddSupportedMimeType ("text/html");
+			RegisterSupportedTypes ();
 			SnippetMode = true;
 		}
 
-		static bool NodeIsHot (String nodeName) 
+		protected bool NodeIsHot (String nodeName) 
 		{
 			return nodeName == "b"
 				|| nodeName == "u"
@@ -54,10 +56,12 @@ namespace Beagle.Filters {
 				|| nodeName == "h3"
 				|| nodeName == "h4"
 				|| nodeName == "h5"
-				|| nodeName == "h6";
+				|| nodeName == "h6"
+				|| nodeName == "i"
+				|| nodeName == "th";
 		}
 
-		static bool NodeBreaksText (String nodeName) 
+		protected static bool NodeBreaksText (String nodeName) 
 		{
 			return nodeName == "td"
 				|| nodeName == "a"
@@ -65,7 +69,7 @@ namespace Beagle.Filters {
 				|| nodeName == "option";
 		}
 
-		static bool NodeBreaksStructure (string nodeName)
+		protected static bool NodeBreaksStructure (string nodeName)
 		{
 			return nodeName == "p"
 				|| nodeName == "br"
@@ -75,17 +79,16 @@ namespace Beagle.Filters {
 				|| nodeName == "h4"
 				|| nodeName == "h5"
 				|| nodeName == "h6";
-
 		}
 		
-		static bool NodeIsContentFree (String nodeName) 
+		protected static bool NodeIsContentFree (String nodeName) 
 		{
 			return nodeName == "script"
 				|| nodeName == "map"
 				|| nodeName == "style";
 		}
 		
-		String WalkChildNodesForText (HtmlNode node)
+		protected String WalkChildNodesForText (HtmlNode node)
 		{
 			StringBuilder builder = new StringBuilder ("");
 			foreach (HtmlNode subnode in node.ChildNodes) {
@@ -94,7 +97,7 @@ namespace Beagle.Filters {
 					if (! NodeIsContentFree (subnode.Name)) {
 						String subtext = WalkChildNodesForText (subnode);
 						builder.Append (subtext);
-			}
+					}
 					break;
 					
 				case HtmlNodeType.Text:
@@ -107,7 +110,7 @@ namespace Beagle.Filters {
 			return builder.ToString ().Trim ();
 		}
 		
-		void WalkHeadNodes (HtmlNode node)
+		protected void WalkHeadNodes (HtmlNode node)
 		{
 			foreach (HtmlNode subnode in node.ChildNodes) {
 				if (subnode.NodeType == HtmlNodeType.Element
@@ -116,10 +119,17 @@ namespace Beagle.Filters {
 					title = HtmlEntity.DeEntitize (title);
 					AddProperty (Beagle.Property.New ("dc:title", title));
 				}
+				if (subnode.NodeType == HtmlNodeType.Element
+				    && subnode.Name == "meta") {
+	   				string name = subnode.GetAttributeValue ("name", "");
+           				string content = subnode.GetAttributeValue ("content", "");
+					if (name != "" && content != "")
+						AddProperty (Beagle.Property.New (name, content));
+				}
 			}
 		}
-		
-		void WalkBodyNodes (HtmlNode node)
+	
+		protected void WalkBodyNodes (HtmlNode node)
 		{
 			switch (node.NodeType) {
 				
@@ -133,6 +143,18 @@ namespace Beagle.Filters {
 						HotUp ();
 					if (breaksText)
 						AppendWhiteSpace ();
+					if (node.Name == "img") {
+						string attr = node.GetAttributeValue ("alt", "");
+						if (attr != "") {
+							AppendText (attr);
+						}
+					}
+					if (node.Name == "a") {
+						string attr = node.GetAttributeValue ("href", "");
+						if (attr != "") {
+							AppendText (attr);
+						}
+					}
 					foreach (HtmlNode subnode in node.ChildNodes)
 						WalkBodyNodes (subnode);
 					if (breaksText)
@@ -153,8 +175,8 @@ namespace Beagle.Filters {
 				
 			}
 		}
-		
-		void WalkNodes (HtmlNode node)
+	
+		protected void WalkNodes (HtmlNode node)
 		{
 			foreach (HtmlNode subnode in node.ChildNodes) {
 				if (subnode.NodeType == HtmlNodeType.Element) {
@@ -166,6 +188,7 @@ namespace Beagle.Filters {
 						WalkHeadNodes (subnode);
 						break;
 					case "body":
+					default:
 						WalkBodyNodes (subnode);
 						break;
 					}
@@ -176,12 +199,23 @@ namespace Beagle.Filters {
 		override protected void DoOpen (FileInfo info)
 		{
 			HtmlDocument doc = new HtmlDocument ();
-			doc.Load (Stream);
+	
+			try {
+				doc.Load (Stream);
+			} catch (NotSupportedException e) {
+				doc.Load (Stream, Encoding.ASCII);
+			}
+
 			if (doc != null)
 				WalkNodes (doc.DocumentNode);
 			Finished ();
 		}
-		
+
+
+		virtual protected void RegisterSupportedTypes () 
+		{
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("text/html"));
+		}
 	}
 	
 }
