@@ -24,11 +24,13 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
-
 using System;
 using System.Collections;
 using System.IO;
 using System.Text;
+
+using Beagle.Daemon;
+
 namespace Beagle.Filters {
 
 	public class FilterPerl : FilterSource {
@@ -45,8 +47,7 @@ namespace Beagle.Filters {
 		
 		public FilterPerl ()
 		{
-			AddSupportedMimeType ("text/x-perl");
-
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("text/x-perl"));
 		}
 
 		override protected void DoOpen (FileInfo info)
