@@ -24,17 +24,18 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
-
 using System;
 using System.IO;
 
+using Beagle.Daemon;
+
 namespace Beagle.Filters {
 
 	public class FilterText : Beagle.Daemon.Filter {
 
 		public FilterText ()
 		{
-			AddSupportedMimeType ("text/plain");
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("text/plain"));
 			SnippetMode = true;
 			OriginalIsText = true;
 		}
