@@ -39,91 +39,109 @@ namespace Beagle.Tile {
 		    Type="MailMessage")]
 	public class TileMailMessage : TileFromHitTemplate {
 
-#if ENABLE_EVO_SHARP
-		string aim_name;
-		string groupwise_name;
-		string icq_name;
-		string jabber_name;
-		string msn_name;
-		string yahoo_name;
-
-		static bool ebook_failed = false;
-#endif
-
-		public TileMailMessage (Hit _hit) : base (_hit,
-							  "template-mail-message.html")
+		public TileMailMessage (Hit _hit) : base (_hit, "template-mail-message.html")
 		{
 		}
 
+		private static string GetHitProperty (Hit hit, string name)
+		{
+			// FIXME: We should handle this case better, but
+			// for now, if we match an attachment, we just want
+			// to display the properties for the parent message.
+			if (hit.ParentUri == null)
+				return hit [name];
+			else
+				return hit ["parent:" + name];
+		}
+
+		private string GetMailIcon ()
+		{
+			string icon;
+			
+			if (GetHitProperty (Hit, "fixme:isAnswered") != null)
+				icon = Images.GetHtmlSourceForStock ("stock_mail-replied", 48);
+			else if (GetHitProperty (Hit, "fixme:isSeen") != null)
+				icon = Images.GetHtmlSourceForStock ("stock_mail-open", 48);
+			else
+				icon = Images.GetHtmlSourceForStock ("stock_mail", 48);
+
+			return icon;
+		}
+
 		protected override void PopulateTemplate ()
 		{
 			base.PopulateTemplate ();
 
-                        bool sent = (Hit ["fixme:isSent"] != null);
+			bool sent = (GetHitProperty (Hit, "fixme:isSent") != null);
+			string str = GetHitProperty (Hit, "dc:title");
 
-			string str;
-
-			str = Hit ["fixme:subject"];
 			if (str == null)
 				str = String.Format ("<i>{0}</i>", Catalog.GetString ("No Subject"));
-			if (Hit ["_IsDeleted"] != null)
-				str = "<strike>" + str + "</strike>";
-			Template["Subject"] = str;
 
+			if (GetHitProperty (Hit, "_IsDeleted") != null)
+				str = "<strike>" + str + "</strike>";
+
+			Template["Icon"] = GetMailIcon ();
+			Template["Subject"] = str;
 			Template["ToFrom"] = sent ? Catalog.GetString ("To") : Catalog.GetString ("From");
 
+			// Limit the number of recipients to 3, so the
+			// tile doesn't look terrible.
 			if (sent) {
-				// Limit the number of recipients to 3, so the
-				// tile doesn't look terrible.
-				ICollection list = InternetAddress.ParseString (Hit ["fixme:to"]);
+				string[] values = Hit.GetProperties ("fixme:to");
 
-				if (list.Count <= 3)
-					Template["Who"] = Hit["fixme:to"];
-				else {
+				if (values != null) {
 					StringBuilder sb = new StringBuilder ();
+					int i;
 
-					int count = 0;
-					foreach (InternetAddress ia in list) {
-						sb.Append (ia.ToString (false));
-						sb.Append (", ");
+					for (i = 0; i < 3 && i < values.Length; i++) {
+						if (i != 0)
+							sb.Append (", ");
 
-						++count;
-						if (count == 3)
-							break;
+						sb.Append (values [i]);
 					}
 
-					sb.Append ("et al");
+					if (i < values.Length)
+						sb.Append (", ...");
 
 					Template["Who"] = sb.ToString ();
 				}
 			} else
-				Template["Who"] = Hit ["fixme:from"];
+				Template["Who"] = GetHitProperty (Hit, "fixme:from");
 
-			Template["Folder"] = Hit ["fixme:folder"];
-			Template["Account"] = Hit ["fixme:account"];
+			Template["Folder"] = GetHitProperty (Hit, "fixme:folder");
+			Template["Account"] = GetHitProperty (Hit, "fixme:account");
 			Template["SentReceived"] = sent ? Catalog.GetString ("Sent") : Catalog.GetString ("Received");
-			Template["When"] = sent ? Hit ["fixme:sentdate"] : Hit ["fixme:received"];
+			Template["When"] = sent ? GetHitProperty (Hit, "fixme:sentdate") : GetHitProperty (Hit, "fixme:received");
 
-			string icon;
-			if (Hit ["fixme:isAnswered"] != null)
-				icon = Images.GetHtmlSourceForStock ("stock_mail-replied", 48);
-			else if (Hit ["fixme:isSeen"] != null)
-				icon = Images.GetHtmlSourceForStock ("stock_mail-open", 48);
-			else
-				icon = Images.GetHtmlSourceForStock ("stock_mail", 48);
-
-			Template["Icon"] = icon;
-			if (Hit ["fixme:isFlagged"] != null)
+			// FIXME: Gross attachment rendering
+			if (Hit.ParentUri != null) {
+				Template["Subject"] = Hit ["fixme:attachment_title"] + " [" + Catalog.GetString ("Email attachment") + "]";
+				Template["EmailSubject"] = str;
+				Gtk.IconSize size = (Gtk.IconSize) 48;
+				string path = BU.GnomeIconLookup.LookupMimeIcon (Hit.MimeType, size);
+				Template["Icon"] = Images.GetHtmlSource (path, BU.GnomeIconLookup.GetFileMimeType (path));
+			}
+			
+			if (GetHitProperty (Hit, "fixme:isFlagged") != null)
 				Template["FollowupIcon"] = Images.GetHtmlSourceForStock ("stock_mail-priority-high", 16);
-			if (Hit ["fixme:hasAttachments"] != null)
+			if (GetHitProperty (Hit, "fixme:hasAttachments") != null)
 				Template["AttachmentIcon"] = Images.GetHtmlSourceForStock ("stock_attach", 16);
 
+#if ENABLE_EVO_SHARP
 			GetImNames (Template["Who"]);
 
-#if ENABLE_EVO_SHARP
 			if (aim_name != null)
 				Template["CanSendIm"] = "";
 #endif
+
+#if ENABLE_GALAGO
+			if (aim_name != null) {
+				string status = BU.GalagoTools.GetPresence ("aim", aim_name);
+				if (status != null && status != "")
+					Template ["Presence"] = status;
+			}
+#endif
 		}
 
 		private string GetEmail (string who)
@@ -139,9 +157,22 @@ namespace Beagle.Tile {
 			return who;
 		}
 
+
+#if ENABLE_EVO_SHARP
+		private string aim_name;
+		private string groupwise_name;
+		private string icq_name;
+		private string jabber_name;
+		private string msn_name;
+		private string yahoo_name;
+
+		static bool ebook_failed = false;
+
 		private void GetImNames (string who)
 		{
-#if ENABLE_EVO_SHARP
+			if (who == null || who == "")
+				return;
+
 			Evolution.Book addressbook = null;
 
 			if (ebook_failed)
@@ -185,30 +216,37 @@ namespace Beagle.Tile {
 				if (c.ImGroupwise.Length > 0)
 					groupwise_name = c.ImGroupwise[0];
 			}
-#endif
 		}
+#endif
 
 		[TileAction]
 		public override void Open () 
 		{
+			string uri_str;
+
 			Process p = new Process ();
 			p.StartInfo.UseShellExecute = false;
 			p.StartInfo.FileName = "evolution";
-			p.StartInfo.Arguments = "'" + Hit.Uri + "'";
+
+			if (Hit.ParentUriAsString != null)
+				uri_str = Hit.ParentUriAsString;
+			else
+				uri_str = Hit.UriAsString;
+
+			p.StartInfo.Arguments  = "'" + uri_str + "'";
 
 			try {
 				p.Start ();
 			} catch (System.ComponentModel.Win32Exception e) {
 				Console.WriteLine ("Unable to run {0}: {1}", p.StartInfo.FileName, e.Message);
 			}
-				
 		}
 
 		[TileAction]
 		public void Mail ()
 		{
-                        bool sent = (Hit ["fixme:isSent"] != null);
-			string address = sent ? Hit ["fixme:to"] : Hit ["fixme:from"];
+                        bool sent = (GetHitProperty (Hit, "fixme:isSent") != null);
+			string address = sent ? GetHitProperty (Hit, "fixme:to") : GetHitProperty (Hit, "fixme:from");
 			
 			Process p = new Process ();
 			p.StartInfo.UseShellExecute = false;
@@ -222,6 +260,30 @@ namespace Beagle.Tile {
 			}
 		}
 
+		[TileAction]
+		public void Reply ()
+		{
+			string uri_str;
+
+			Process p = new Process ();
+			p.StartInfo.UseShellExecute = false;
+			p.StartInfo.FileName = "evolution";
+
+			if (Hit.ParentUriAsString != null)
+				uri_str = Hit.ParentUriAsString;
+			else
+				uri_str = Hit.UriAsString;
+			
+			p.StartInfo.Arguments = String.Format ("'{0};reply=sender'", uri_str);
+
+			try {
+				p.Start ();
+			} catch (System.ComponentModel.Win32Exception e) {
+				Console.WriteLine ("Unable to run {0}: {1}", p.StartInfo.FileName, e.Message);
+			}
+				
+		}			
+
 #if ENABLE_EVO_SHARP
 		[TileAction]
 		public void SendIm () 
