@@ -43,10 +43,12 @@ namespace Beagle.Util {
 		public long   LogOffset;
 
 		public string Protocol;
+		public string Client;
 
 		public DateTime StartTime;
 		public DateTime EndTime;
 		public DateTime Timestamp;
+		public string   Snippet;
 
 		public string SpeakingTo;
 		public string Identity;
@@ -62,27 +64,30 @@ namespace Beagle.Util {
 
 		//////////////////////////
 
-		protected ImLog (string protocol, string file, long offset)
+		protected ImLog (string client, string protocol, string file, long offset)
 		{
+			Client = client;
 			Protocol = protocol;
 			LogFile = file;
 			LogOffset = offset;
 		}
 
-		protected ImLog (string protocol, string file) : this (protocol, file, -1)
+		protected ImLog (string client, string protocol, string file) : this (client, protocol, file, -1)
 		{ }
 
 		public Uri Uri {
-			get {
-				// Hacky
-				string when = StartTime.ToString ("yyyy.MM.dd.HH.mm.ss");
-				string uriStr = String.Format ("imlog://{0}/{1}/{2}/{3}",
-							       Protocol, Identity, SpeakingTo,
-							       when);
-				return new Uri (uriStr, true);
-			}							    
+			get { return UriFu.PathToFileUri (this.LogFile); }
 		}
 
+		public string EllipsizedSnippet {
+			get {
+				string snippet = Snippet;
+				// FIXME: We should try to avoid breaking mid-word
+				if (snippet != null && snippet.Length > 50)
+					snippet = snippet.Substring (0, 50) + "...";
+				return snippet;
+			}
+		}
 
 		public ICollection Speakers {
 			get { return speakerHash.Keys; }
@@ -98,6 +103,10 @@ namespace Beagle.Util {
 			}
 		}
 
+		protected IList RawUtterances {
+			get { return utterances; }
+		}
+
 		protected void AddUtterance (DateTime timestamp,
 					     String   who,
 					     String   text)
@@ -126,6 +135,11 @@ namespace Beagle.Util {
 			}
 		}
 
+		protected void ClearUtterances ()
+		{
+			utterances.Clear ();
+		}
+
 		protected abstract void Load ();
 	}
 
@@ -139,23 +153,43 @@ namespace Beagle.Util {
 
 		private static string StripTags (string line)
 		{
-			int i, j;
-			while (true) {
-				i = line.IndexOf ('<');
-
-				if (i == -1)
+			int first = line.IndexOf ('<');
+			if (first == -1)
+				return line;
+			
+			StringBuilder builder = new StringBuilder ();
+			int i = 0;
+			while (i < line.Length) {
+				
+				int j;
+				if (first == -1) {
+					j = line.IndexOf ('<', i);
+				} else {
+					j = first;
+					first = -1;
+				}
+				
+				int k = -1;
+				if (j != -1) {
+					k = line.IndexOf ('>', j);
+					
+					// If a "<" is unmatched, preserve it, and the
+					// rest of the line
+					if (k == -1)
+						j = -1;
+				}
+				
+				if (j == -1) {
+					builder.Append (line, i, line.Length - i);
 					break;
-
-				j = line.IndexOf ('>', i);
-
-				// If a "<" is unmatched, preserve it, and the
-				// rest of the line
-                                if (j == -1)
-					break;
-                               
-				line = line.Substring (0, i) + line.Substring (j+1);
+				}
+				
+				builder.Append (line, i, j-i);
+				
+				i = k+1;
 			}
-			return line;
+			
+			return builder.ToString ();
 		}
 
 		private static bool IsNewConversation (string line)
@@ -201,23 +235,61 @@ namespace Beagle.Util {
 
 		///////////////////////////////////////
 
-		private GaimLog (string file, long offset) : base ("gaim", file, offset)
-		{ }
+		private bool TrySnippet ()
+		{
+			int best_word_count = 0;
 
-		private GaimLog (string file) : base ("gaim", file)
-		{ }
+			foreach (Utterance utt in RawUtterances) {
+
+				string possible_snippet = utt.Text.Trim ();
+
+				int word_count = StringFu.CountWords (possible_snippet, 15);
+				if (word_count > best_word_count) {
+					Snippet = possible_snippet;
+					best_word_count = word_count;
+				}
 
 
-		private void ProcessLine (string line)
+				if (word_count > 3)
+					return true;
+			}
+
+			return false;
+		}
+
+		// FIXME: The ending timestamp in the log will be inaccurate
+		// until Load is called... before that, the ending time will
+		// come from the timestamp of the snippet-line.
+
+		private void SetSnippet ()
+		{
+			LoadWithTermination (new LoadTerminator (TrySnippet));
+		}
+
+		///////////////////////////////////////
+
+		private GaimLog (string protocol, string file, long offset) : base ("gaim", protocol, file, offset)
+		{ 
+			SetSnippet ();
+		}
+
+		private GaimLog (string protocol, string file) : base ("gaim", protocol, file)
+		{ 
+			SetSnippet ();
+		}
+
+		// Return true if a new utterance is now available,
+		// and false if the previous utterance was changed.
+		private bool ProcessLine (string line)
 		{
 			if (! line.StartsWith ("(")) {
 				AppendToPreviousUtterance (line);
-				return;
+				return false;
 			}
 			int j = line.IndexOf (')');
 			if (j == -1) {
 				AppendToPreviousUtterance (line);
-				return;
+				return false;
 			}
 			string whenStr = line.Substring (1, j-1);
 			string[] whenSplit = whenStr.Split (':');
@@ -230,7 +302,7 @@ namespace Beagle.Util {
 				// If something goes wrong, this line probably
 				// spills over from the previous one.
 				AppendToPreviousUtterance (line);
-				return;
+				return false;
 			}
 
 			line = line.Substring (j+1).Trim ();
@@ -248,14 +320,24 @@ namespace Beagle.Util {
 
 			int i = line.IndexOf (':');
 			if (i == -1)
-				return;
+				return false;
 			string alias = line.Substring (0, i);
 			string text = line.Substring (i+1).Trim ();
 
 			AddUtterance (when, alias, text);
+
+			return true;
 		}
 
+		protected delegate bool LoadTerminator ();
+
 		protected override void Load ()
+		{
+			ClearUtterances ();
+			LoadWithTermination (null);
+		}
+
+		protected void LoadWithTermination (LoadTerminator terminator)
 		{
 			FileStream fs;
 			StreamReader sr;
@@ -281,7 +363,7 @@ namespace Beagle.Util {
 
 				// Could the second line ever start w/ < in a non-html log?
 				// I hope not!
-				bool isHtml = line.StartsWith ("<");
+				bool isHtml = line.Length > 0 && line [0] == '<';
 				
 				while ((line = sr.ReadLine ()) != null) {
 					if (isHtml)
@@ -289,8 +371,13 @@ namespace Beagle.Util {
 				
 					if (IsNewConversation (line))
 						break;
-
-					ProcessLine (line);
+					
+					// Only check termination when a new Utterance has become
+					// available.
+					if (ProcessLine (line)
+					    && terminator != null
+					    && terminator ())
+						break;
 				}
 			}
 
@@ -300,18 +387,25 @@ namespace Beagle.Util {
 
 		private static void ScanNewStyleLog (FileInfo file, ArrayList array)
 		{
-			ImLog log = new GaimLog (file.FullName);
-			
-			log.Timestamp = file.LastWriteTime;
-			
+			// file.Directory.Parent.Parent.Name is the name of the current protocol (ex. aim)
+			ImLog log = new GaimLog (file.Directory.Parent.Parent.Name, file.FullName);
+
 			string startStr = Path.GetFileNameWithoutExtension (file.Name);
-			log.StartTime = DateTime.ParseExact (startStr,
-							     "yyyy-MM-dd.HHmmss",
-							     CultureInfo.CurrentCulture);
-
-			log.SpeakingTo = file.Directory.Name;
-			log.Identity   = file.Directory.Parent.Name;
+			try {
+				log.StartTime = DateTime.ParseExact (startStr,
+								     "yyyy-MM-dd.HHmmss",
+								     CultureInfo.CurrentCulture);
+			} catch (FormatException) {
+				Logger.Log.Warn ("IMLog: Could not parse date/time from '{0}', ignoring.", startStr);
+				return;
+			}
+			
+			log.Timestamp = file.LastWriteTime;	
 
+			// Gaim likes to represent many characters in hex-escaped %xx form
+			log.SpeakingTo = StringFu.HexUnescape (file.Directory.Name);
+			log.Identity   = StringFu.HexUnescape (file.Directory.Parent.Name);
+			
 			array.Add (log);
 		}
 
@@ -338,7 +432,7 @@ namespace Beagle.Util {
 				if (isHtml)
 					line = StripTags (line);
 				if (IsNewConversation (line)) {
-					ImLog log = new GaimLog (file.FullName, offset);
+					ImLog log = new GaimLog ("aim", file.FullName, offset); //FIXME: protocol
 					log.StartTime = NewConversationTime (line);
 					log.Identity = "_OldGaim_"; // FIXME: parse a few lines of the log to figure this out
 					log.SpeakingTo = speakingTo;
