@@ -42,6 +42,10 @@ namespace Beagle.Util {
 			// We used to support the EXERCISE_THE_DOG env variable, but
 			// that was dropped.  
 			exercise_the_dog = Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG");
+			if (exercise_the_dog == null &&
+			    Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG_HARDER") != null)
+				exercise_the_dog = "harder fallback";
+
 			if (exercise_the_dog != null) {
 				
 				global_delay = 0.0;
@@ -421,6 +425,15 @@ namespace Beagle.Util {
 					if (immediate_priority_only && task.Priority != Priority.Immediate)
 						return false;
 
+					// Keep track of when immediate priority tasks are
+					// added so that we can throttle if the scheduler
+					// is being slammed with them.
+					if (task.Priority == Priority.Immediate) {
+						// Shift our times down by one
+						Array.Copy (last_immediate_times, 1, last_immediate_times, 0, 4);
+						last_immediate_times [4] = DateTime.Now;
+					}
+
 					old_task = task_by_tag [task.Tag] as Task;
 					if (old_task == task)
 						return true;
@@ -562,6 +575,8 @@ namespace Beagle.Util {
 		const double default_idle_rate_factor    = 2.097; // work about 1/3rd of the time
 		const double default_maximum_delay       = 20;    // never wait for more than 20s
 
+		DateTime[] last_immediate_times = new DateTime [5];
+
 		private double GetIdleTime ()
 		{
 			return SystemInformation.InputIdleTime;
@@ -582,7 +597,10 @@ namespace Beagle.Util {
 			double idle_time = GetIdleTime ();
 			double idle_scale = 1.0;
 			bool is_idle = false;
-			if (idle_time > idle_threshold) {
+			bool need_throttle = false;
+
+			// Never speed up if we are using the battery.
+			if (idle_time > idle_threshold && ! SystemInformation.UsingBattery) {
 				is_idle = true;
 				double t = (idle_time - idle_threshold) / idle_ramp_up_time;				     
 				idle_scale = (1 - Math.Min (t, 1.0));
@@ -592,6 +610,28 @@ namespace Beagle.Util {
 				
 			case Priority.Immediate:
 				rate_factor = 0;
+
+				if (last_immediate_times [0] != DateTime.MinValue) {
+					TimeSpan last_add_delta = DateTime.Now.Subtract (last_immediate_times [4]);
+
+					// If less than a second has gone by since the
+					// last immediate task was added, there is
+					// still a torrent of events coming in, and we
+					// may need to throttle.
+					if (last_add_delta.Seconds <= 1) {
+						TimeSpan between_add_delta = last_immediate_times [4].Subtract (last_immediate_times [0]);
+
+						// At least 5 immediate tasks have been
+						// added in the last second.  We
+						// definitely need to throttle.
+						if (between_add_delta.Seconds <= 1) {
+							Logger.Log.Debug ("Thottling immediate priority tasks");
+							need_throttle = true;
+							rate_factor = idle_scale * default_idle_rate_factor;
+						}
+					}
+				}
+
 				break;
 
 			case Priority.Generator:
@@ -624,6 +664,11 @@ namespace Beagle.Util {
 			if (delay > default_maximum_delay)
 				delay = default_maximum_delay;
 
+			// If we need to throttle, make sure we don't delay less than
+			// a second and some.
+			if (need_throttle && delay < 1.25)
+				delay = 1.25;
+
 			return delay;
 		}
 
