@@ -183,7 +183,7 @@ namespace Beagle.Daemon.EvolutionDataServerQueryable {
 			string rev_str = ExtractFieldFromVCardString (vcard_str, "REV");
 			DateTime rev = RevStringToDateTime (rev_str);
 
-			if (rev < IndexedThrough)
+			if (rev != DateTime.MinValue && rev < IndexedThrough)
 				return null;
 
 			Indexable indexable = new Indexable (GetContactUri (contact));
@@ -239,12 +239,13 @@ namespace Beagle.Daemon.EvolutionDataServerQueryable {
 
 			if (photo.Data != null && photo.Data.Length > 0) {
 				string photo_filename = GetPhotoFilename (contact.Id);
-				Stream s = File.OpenWrite (photo_filename);
+				Stream s = new FileStream (photo_filename, FileMode.Create, FileAccess.Write, FileShare.ReadWrite);
 				BinaryWriter w = new BinaryWriter (s);
 				w.Write (photo.Data);
 				w.Close ();
+				s.Close ();
 
-				indexable.AddProperty (Property.NewUnsearched ("Photo", photo_filename));
+				indexable.AddProperty (Property.NewUnsearched ("beagle:Photo", photo_filename));
 			}
 			// FIXME: List?
 			// FIXME: ListShowAddresses?
