@@ -31,11 +31,9 @@ using System.Reflection;
 using System.Text;
 using System.Threading;
 
-
 using Beagle.Daemon;
 using Beagle.Util;
 
-
 namespace Beagle.Daemon.FileSystemQueryable {
 
 	[QueryableFlavor (Name="Files", Domain=QueryDomain.Local, RequireInotify=false)]
@@ -249,8 +247,46 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			}
 		}
 
-		//////////////////////////////////////////////////////////////////////////
+		override protected void AbusiveChildIndexableHook (Indexable child_indexable)
+		{
+			// FIXME: FileSystemQueryable does not support adding children 
+			// to the NameIndex at the moment, this, however, would be nice 
+			// to have if we are to index compressed archives and such.
+			throw new InvalidOperationException ();
 
+			if (Debug)
+				Logger.Log.Debug ("AbusiveChildIndexableHook: uri={0}", child_indexable.Uri);
+
+			Uri internal_uri = model.ToInternalUri (child_indexable.Uri);
+
+			if (Debug)
+				Logger.Log.Debug ("AbusiveChildIndexableHook: Remapped '{0}' to '{1}'", child_indexable.Uri, internal_uri);
+
+			child_indexable.Uri = internal_uri;
+		}
+
+		override protected void AbusiveUriFilteredHook (FilteredStatus uri_filtered)
+		{
+			if (!model.InternalUriIsValid (uri_filtered.Uri)) {
+				Logger.Log.Error ("AbusiveUriFilteredHook: Internal uri is not valid: {0}", uri_filtered.Uri);
+				return;
+			}
+
+			Uri external_uri = model.FromInternalUri (uri_filtered.Uri);
+			
+			if (Debug)
+				Logger.Log.Debug ("AbusiveUriFilteredHook: external_uri={0} internal_uri={1}, name={2}, version={3}", external_uri, uri_filtered.Uri, uri_filtered.FilterName, uri_filtered.FilterVersion);
+
+			try {
+				model.MarkAsFiltered (external_uri.LocalPath, uri_filtered.FilterName, uri_filtered.FilterVersion);
+			} catch (Exception ex) {
+				Logger.Log.Error ("Could not mark file filtered");
+				Logger.Log.Error (ex);
+			}
+		}
+		
+		//////////////////////////////////////////////////////////////////////////
+		
 		// Filter out hits where the files seem to no longer exist.
 		override protected bool HitIsValid (Uri uri)
 		{
@@ -275,7 +311,9 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		public void StartWorker ()
 		{
 			event_backend.Start (this);
-			Conf.Subscribe (typeof (Conf.IndexingConfig), model.LoadConf);
+
+			model.LoadConfiguration ();
+
 			log.Info ("FileSystemQueryable start-up thread finished");
 			// FIXME: Do we need to re-run queries when we are fully started?
 		}
