@@ -37,20 +37,6 @@ namespace Beagle.Daemon {
 
 	public class Filter {
 
-		public enum ContentType {
-			Text,
-			WhiteSpace,
-			StructuralBreak
-		}
-
-		public delegate void GotPropertyHandler (Property prop);
-		public delegate void GotContentHandler  (ContentType type, string content);
-
-		public event GotPropertyHandler GotPropetyEvent;
-		public event GotContentHandler  GotContentEvent;
-
-		//////////////////////////
-
 		// Derived classes always must have a constructor that
 		// takes no arguments.
 		public Filter () { }
@@ -66,27 +52,26 @@ namespace Beagle.Daemon {
 
 		//////////////////////////
 
-		private   ArrayList supported_mime_types = new ArrayList ();
-		private   ArrayList supported_extensions = new ArrayList ();
+		private bool delete_content;
+
+		public bool DeleteContent {
+			get { return this.delete_content; }
+			set { this.delete_content = value; }
+		}
+
+		//////////////////////////
+
+		private ArrayList supported_flavors = new ArrayList ();
 		
-		protected void AddSupportedMimeType (string mime_type)
+		protected void AddSupportedFlavor (FilterFlavor flavor) 
 		{
-			supported_mime_types.Add (mime_type);
+			supported_flavors.Add (flavor);
 		}
 
-		protected void AddSupportedExtension (string extension)
-		{
-			supported_extensions.Add (extension);
+		public ICollection SupportedFlavors {
+			get { return supported_flavors; }
 		}
-
-		public IEnumerable SupportedMimeTypes {
-			get { return supported_mime_types; }
-		}
-
-		public IEnumerable SupportedExtensions {
-			get { return supported_mime_types; }
-		}
-
+		
 		//////////////////////////
 
 		// Filters are versioned.  This allows us to automatically re-index
@@ -119,6 +104,8 @@ namespace Beagle.Daemon {
 			version = v;
 		}
 
+		
+
 		//////////////////////////
 
 		private string this_mime_type = null;
@@ -217,23 +204,57 @@ namespace Beagle.Daemon {
 
 		private bool last_was_structural_break = true;
 
+		// This two-arg AppendText() will give flexibility to
+		// filters to segregate hot-contents and
+		// normal-contents of a para and call this method with
+		// respective contents.  
+		//
+		// str : Holds both the normal-contents and hot contents.
+		// strHot: Holds only hot-contents.
+		//
+		// Ex:- suppose the actual-content is "one <b>two</b> three"
+		// str = "one two three"
+		// strHot = "two"
+		//
+		// NOTE: HotUp() or HotDown() has NO-EFFECT on this variant 
+		// of AppendText ()
+		
+		public void AppendText (string str, string strHot)
+		{
+			if (!IsFrozen && str != null && str != "") {
+				
+				// FIXME: Do we need to handle '\n' in any 
+				// other way?
+				int i = 0;
+				string line;
+				string[] lines = str.Split ('\n');
+				for (i = 0; i < lines.Length; i++) {
+					line = lines[i].Trim();
+					if (line.Length > 0) {
+						ReallyAppendText (line, null);
+						AppendStructuralBreak ();
+					}
+				}
+			}
+			ReallyAppendText (null, strHot);
+		}
+		
 		public void AppendText (string str)
 		{
 			//Logger.Log.Debug ("AppendText (\"{0}\")", str);
-			if (! IsFrozen && str != null && str != "") {
+			if (! IsFrozen && str != null && str != "")
+				AppendText (str, IsHot ? str : null);
+		}
 
-				// FIXME: We should be smarter about newlines
-				if (str.IndexOf ('\n') != -1)
-					str = str.Replace ("\n", " ");
+		// Does adding text to to text/hot pools respectively.
+		private void ReallyAppendText (string str, string strHot)
+		{
+			if (!IsFrozen && strHot != null && strHot != "")
+				hotPool.Add (strHot.Trim()+" ");
 
+			if (str != null) {
 				textPool.Add (str);
 
-				// For Hot pool, Add the "whitespace"
-				// while adding text to it. This way
-				// we can avoid extra spaces.
-				if (IsHot)
-					hotPool.Add (str.Trim()+" ");
-
 				int pool_size = 0;
 				foreach (string x in textPool)
 					pool_size += x.Length;
@@ -253,7 +274,6 @@ namespace Beagle.Daemon {
 				last_was_structural_break = false;
 			}
 		}
-
 		private bool NeedsWhiteSpace (ArrayList array)
 		{
 			if (array.Count == 0)
@@ -312,10 +332,30 @@ namespace Beagle.Daemon {
 			isFinished = true;
 		}
 
+		private bool has_error = false;
+
+		public bool HasError {
+			get { return has_error; }
+		}
+
+		protected void Error ()
+		{
+			has_error = true;
+		}
+
 		//////////////////////////
 
+		protected virtual void DoOpen (FileSystemInfo info) {
+			if (info is FileInfo)
+				DoOpen (info as FileInfo);
+			else if (info is DirectoryInfo)
+				DoOpen (info as DirectoryInfo);
+		}
+
 		protected virtual void DoOpen (FileInfo info) { }
 
+		protected virtual void DoOpen (DirectoryInfo info) { }
+
 		protected virtual void DoPullProperties () { }
 
 		protected virtual void DoPullSetup () { }
@@ -340,13 +380,12 @@ namespace Beagle.Daemon {
 		  
 		*/
 
-		private bool isOpen = false;
 		private string tempFile = null;
-		private FileInfo currentInfo = null;
+		private FileSystemInfo currentInfo = null;
 		private FileStream currentStream = null;
 		private StreamReader currentReader = null;
 
-		public void Open (Stream stream)
+		public bool Open (Stream stream)
 		{
 			// If we are handed a stream, dump it into
 			// a temporary file.
@@ -362,10 +401,10 @@ namespace Beagle.Daemon {
 
 			tempStream.Close ();
 
-			Open (new FileInfo (tempFile));
+			return Open (new FileInfo (tempFile));
 		}
 
-		public void Open (FileInfo info)
+		public bool Open (FileSystemInfo info)
 		{
 			isFinished = false;
 			textPool = new ArrayList ();
@@ -374,52 +413,74 @@ namespace Beagle.Daemon {
 
 			currentInfo = info;
 
-			// Open a stream for this file.
-			currentStream = new FileStream (info.FullName,
-							FileMode.Open,
-							FileAccess.Read,
-							FileShare.Read);
-
-			// Our default assumption is sequential reads.
-			// FIXME: Is this the right thing to do here?
-			FileAdvise.IncreaseReadAhead (currentStream);
-
-			// Give the OS a hint that we will be reading this
-			// file soon.
-			FileAdvise.PreLoad (currentStream);			
-
-			DoOpen (info);
-			isOpen = true;
-
-			if (IsFinished) {
-				isOpen = false;
-				return;
-			}
-			
-			DoPullProperties ();
-			if (IsFinished) {
-				isOpen = false;
-				return;
+			if (info is FileInfo) {
+				// Open a stream for this file.
+				currentStream = new FileStream (info.FullName,
+								FileMode.Open,
+								FileAccess.Read,
+								FileShare.Read);
+				
+				// Our default assumption is sequential reads.
+				// FIXME: Is this the right thing to do here?
+				FileAdvise.IncreaseReadAhead (currentStream);
+				
+				// Give the OS a hint that we will be reading this
+				// file soon.
+				FileAdvise.PreLoad (currentStream);				
 			}
 
-			// Close and reset our TextReader
-			if (currentReader != null) {
-				currentReader.Close ();
-				currentReader = null;
+			try {
+				DoOpen (info);
+
+				if (IsFinished)
+					return true;
+				else if (HasError)
+					return false;
+				
+				DoPullProperties ();
+				
+				if (IsFinished) 
+					return true;
+				else if (HasError)
+					return false;
+				
+				// Close and reset our TextReader
+				if (currentReader != null) {
+					currentReader.Close ();
+					currentReader = null;
+				}
+				
+				// Seek back to the beginning of our stream
+				currentStream.Seek (0, SeekOrigin.Begin);
+				
+				DoPullSetup ();
+				
+				if (HasError)
+					return false;				
+			} catch (Exception e) {
+				Logger.Log.Warn ("Unable to filter {0}: {1}", info.FullName, e.Message);
+				return false;
 			}
 
-			// Seek back to the beginning of our stream
-			currentStream.Seek (0, SeekOrigin.Begin);
+			return true;
+		}
 
-			DoPullSetup ();
-			if (IsFinished) {
-				isOpen = false;
-				return;
-			}
+		public bool Open (string path)
+		{
+			if (File.Exists (path))
+				return Open (new FileInfo (path));
+			else if (Directory.Exists (path))
+				 return Open (new DirectoryInfo (path));
+			else 
+				return false;
 		}
 
 		public FileInfo FileInfo {
-			get { return currentInfo; }
+			get { return currentInfo as FileInfo; }
+		}
+
+		public DirectoryInfo DirectoryInfo {
+			get { return currentInfo as DirectoryInfo; }
 		}
 
 		public Stream Stream {
@@ -449,8 +510,6 @@ namespace Beagle.Daemon {
 			return true;
 		}
 
-		private bool closed = false;
-
 		private void Close ()
 		{
 			if (currentStream == null)
@@ -475,6 +534,15 @@ namespace Beagle.Daemon {
 			if (tempFile != null)
 				File.Delete (tempFile);
 
+			if (currentInfo != null && this.DeleteContent) {
+				try {
+					currentInfo.Delete ();
+				} catch (Exception e) {
+					Logger.Log.Debug ("Caught exception trying to delete {0} in filter '{1}'",
+							  currentInfo.FullName, Name);
+					Logger.Log.Debug (e);					
+				}
+			}
 		}
 
 		private string PullFromArray (ArrayList array)
@@ -488,14 +556,27 @@ namespace Beagle.Daemon {
 			return null;
 		}
 
+		private string PullTextCarefully (ArrayList array)
+		{
+			string text = null;
+			try {	
+				text = PullFromArray (array);
+			} catch (Exception ex) {
+				Logger.Log.Debug ("Caught exception while pulling text in filter '{0}'", Name);
+				Logger.Log.Debug (ex);
+			}
+			return text;
+
+		}
+
 		private string PullText ()
 		{
-			return PullFromArray (textPool);
+			return PullTextCarefully (textPool);
 		}
 
 		private string PullHotText ()
 		{
-			return PullFromArray (hotPool);
+			return PullTextCarefully (hotPool);
 		}
 
 		public TextReader GetTextReader ()
@@ -513,5 +594,36 @@ namespace Beagle.Daemon {
 		public IEnumerable Properties {
 			get { return propertyPool; }
 		}
+
+		//////////////////////////////
+
+		// This is used primarily for the generation of URIs for the
+		// child indexables that can be created as a result of the
+		// filtering process.
+
+		private Uri uri = null;
+
+		public Uri Uri {
+			get { return uri; }
+			set { uri = value; }
+		}
+
+		//////////////////////////////
+
+		private ArrayList child_indexables = new ArrayList ();
+
+		protected void AddChildIndexable (Indexable indexable)
+		{
+			this.child_indexables.Add (indexable);
+		}
+		
+		protected void AddChildIndexables (ICollection indexables)
+		{
+			this.child_indexables.AddRange (indexables);
+		}
+
+		public ArrayList ChildIndexables {
+			get { return this.child_indexables; }
+		}
 	}
 }
