@@ -24,7 +24,6 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
-
 using System;
 using System.IO;
 using System.Collections;
@@ -32,7 +31,6 @@ using System.Reflection;
 using System.Text;
 using System.Threading;
 using Beagle.Util;
-
 namespace Beagle.Daemon {
 	
 	public class QueryDriver {
@@ -41,7 +39,7 @@ namespace Beagle.Daemon {
 
 		static ArrayList allowed_queryables = new ArrayList ();
 		static ArrayList denied_queryables = new ArrayList ();
-		
+
 		static public void Allow (string name)
 		{
 			allowed_queryables.Add (name.ToLower ());
@@ -76,6 +74,18 @@ namespace Beagle.Daemon {
 
 		//////////////////////////////////////////////////////////////////////////////////////
 
+		// Paths to static queryables
+
+		static ArrayList static_queryables = new ArrayList ();
+		
+		static public void AddStaticQueryable (string path) {
+
+			if (! static_queryables.Contains (path))
+				static_queryables.Add (path);
+		}
+
+		//////////////////////////////////////////////////////////////////////////////////////
+
 		// Use introspection to find all classes that implement IQueryable, the construct
 		// associated Queryables objects.
 
@@ -108,7 +118,12 @@ namespace Beagle.Daemon {
 						continue;
 
 					if (flavor.RequireInotify && ! Inotify.Enabled) {
-						Logger.Log.Warn ("Can't start backend '{0}' without inotify",  flavor.Name);
+						Logger.Log.Warn ("Can't start backend '{0}' without inotify", flavor.Name);
+						continue;
+					}
+
+					if (flavor.RequireExtendedAttributes && ! ExtendedAttribute.Supported) {
+						Logger.Log.Warn ("Can't start backend '{0}' without extended attributes", flavor.Name);
 						continue;
 					}
 
@@ -132,6 +147,85 @@ namespace Beagle.Daemon {
 			Logger.Log.Debug ("Found {0} types in {1}", count, assembly.FullName);
 		}
 
+		////////////////////////////////////////////////////////
+
+		// Scans PathFinder.SystemIndexesDir after available 
+		// system-wide indexes.
+		static void LoadSystemIndexes () 
+		{
+			if (!Directory.Exists (PathFinder.SystemIndexesDir))
+				return;
+			
+			int count = 0;
+
+			foreach (DirectoryInfo index_dir in new DirectoryInfo (PathFinder.SystemIndexesDir).GetDirectories ()) {
+				if (! UseQueryable (index_dir.Name))
+					continue;
+				
+				if (LoadStaticQueryable (index_dir, QueryDomain.System))
+					count++;
+			}
+
+			Logger.Log.Debug ("Found {0} system-wide indexes", count);
+		}
+
+		// Scans configuration for user-specified index paths 
+		// to load StaticQueryables from.
+		static void LoadStaticQueryables () 
+		{
+			int count = 0;
+
+			foreach (string path in Conf.Daemon.StaticQueryables)
+				static_queryables.Add (path);
+
+			foreach (string path in static_queryables) {
+				DirectoryInfo index_dir = new DirectoryInfo (path);
+
+				if (!index_dir.Exists)
+					continue;
+				
+				// FIXME: QueryDomain might be other than local
+				if (LoadStaticQueryable (index_dir, QueryDomain.Local))
+					count++;
+			}
+
+			Logger.Log.Debug ("Found {0} user-configured static queryables", count);
+		}
+
+		// Instantiates and loads a StaticQueryable from an index directory
+		static private bool LoadStaticQueryable (DirectoryInfo index_dir, QueryDomain query_domain) 
+		{
+			StaticQueryable static_queryable = null;
+			
+			if (!index_dir.Exists)
+				return false;
+			
+			try {
+				static_queryable = new StaticQueryable (index_dir.Name, index_dir.FullName, true);
+			} catch (Exception e) {
+				Logger.Log.Error ("Caught exception while instantiating static queryable: {0}", index_dir.Name);
+				Logger.Log.Error (e);					
+				return false;
+			}
+			
+			if (static_queryable != null) {
+				QueryableFlavor flavor = new QueryableFlavor ();
+				flavor.Name = index_dir.Name;
+				flavor.Domain = query_domain;
+				
+				Queryable queryable = new Queryable (flavor, static_queryable);
+				queryables.Add (queryable);
+				
+				iqueryable_to_queryable [static_queryable] = queryable;
+
+				return true;
+			}
+
+			return false;
+		}
+
+		////////////////////////////////////////////////////////
+
 		static private Type[] GetQueryableTypes (Assembly assembly)
 		{
 			Type[] assembly_types = assembly.GetTypes ();
@@ -160,7 +254,7 @@ namespace Beagle.Daemon {
 
 		static private Assembly[] GetAssemblies ()
 		{
-			Assembly[] assemblies;
+ 			Assembly[] assemblies;
 			int i = 0;
 			DirectoryInfo backends = new DirectoryInfo (PathFinder.BackendDir);
 
@@ -190,17 +284,31 @@ namespace Beagle.Daemon {
 				Server.ScanAssemblyForExecutors (assembly);
 			}
 
+			// FIXME: This is not nice at all
+			if (UseQueryable ("system"))
+				LoadSystemIndexes ();
+			if (UseQueryable ("static"))
+				LoadStaticQueryables ();
+
 			foreach (Queryable q in queryables)
 				q.Start ();
 		}
 
 		static public string ListBackends ()
 		{
-			string ret = "";
+			string ret = "User:\n";
 			foreach (Assembly assembly in GetAssemblies ())
 				foreach (Type type in GetQueryableTypes (assembly))
 					foreach (QueryableFlavor flavor in GetQueryableFlavors (type))
-						ret += " - " + flavor.Name + "\n";
+				ret += String.Format (" - {0}\n", flavor.Name);
+			
+			if (!Directory.Exists (PathFinder.SystemIndexesDir)) 
+				return ret;
+			
+			ret += "System:\n";
+			foreach (DirectoryInfo index_dir in new DirectoryInfo (PathFinder.SystemIndexesDir).GetDirectories ()) {
+				ret += String.Format (" - {0}\n", index_dir.Name);
+			}
 
 			return ret;
 		}
