@@ -39,12 +39,15 @@ namespace Beagle.Daemon {
 		static string helper_path;
 
 		string remote_index_name;
+		int remote_index_minor_version = 0;
 		int last_item_count = -1;
 
 		RemoteIndexerRequest pending_request = new RemoteIndexerRequest ();
 		object pending_request_lock = new object ();
 
 		public event IIndexerChangedHandler ChangedEvent;
+		public event IIndexerChildIndexableHandler ChildIndexableEvent;
+		public event IIndexerUrisFilteredHandler UrisFilteredEvent;
 
 		static RemoteIndexer ()
 		{
@@ -58,14 +61,15 @@ namespace Beagle.Daemon {
 			Logger.Log.Debug ("Found index helper at {0}", helper_path);
 		}
 
-		static public IIndexer NewRemoteIndexer (string name)
+		static public IIndexer NewRemoteIndexer (string name, int minor_version)
 		{
-			return new RemoteIndexer (name);
+			return new RemoteIndexer (name, minor_version);
 		}
 
-		public RemoteIndexer (string name)
+		public RemoteIndexer (string name, int minor_version)
 		{
 			this.remote_index_name = name;
+			this.remote_index_minor_version = minor_version;
 		}
 
 		public void Add (Indexable indexable)
@@ -129,6 +133,7 @@ namespace Beagle.Daemon {
 				start_helper_by_hand = true;
 
 			request.RemoteIndexName = remote_index_name;
+			request.RemoteIndexMinorVersion = remote_index_minor_version;
 			
 			while (response == null
 			       && exception_count < 5
@@ -168,6 +173,21 @@ namespace Beagle.Daemon {
 			if (response != null) {
 				//Logger.Log.Debug ("Got response!");
 				last_item_count = response.ItemCount;
+
+				if (response.ChildIndexables != null) {
+					Logger.Log.Debug ("Sending {0} child indexables from helper",
+							  response.ChildIndexables.Length);
+
+					if (response.ChildIndexables.Length > 0 && ChildIndexableEvent != null)
+						ChildIndexableEvent (response.ChildIndexables);
+				}
+
+				if (response.UrisFiltered != null) {
+					Logger.Log.Debug ("Sending {0} filtered uris from helper", response.UrisFiltered.Length);
+
+					if (response.UrisFiltered.Length > 0 && UrisFilteredEvent != null)
+						UrisFilteredEvent (response.UrisFiltered);
+				}
 			} else if (exception_count >= 5) {
 				Logger.Log.Error ("Exception limit exceeded trying to activate a helper.  Giving up on indexing!");
 			}
