@@ -45,6 +45,8 @@ namespace Beagle.Daemon {
 							 this.driver.Fingerprint);
 
 			this.driver.ChangedEvent += OnLuceneDriverChanged;
+			this.driver.ChildIndexableEvent += OnLuceneChildIndexableEvent;
+			this.driver.UrisFilteredEvent += OnLuceneUrisFilteredEvent;
 		}
 
 		// We assume that
@@ -111,6 +113,8 @@ namespace Beagle.Daemon {
 		}
 
 		public event IIndexerChangedHandler ChangedEvent;
+		public event IIndexerChildIndexableHandler ChildIndexableEvent;
+		public event IIndexerUrisFilteredHandler UrisFilteredEvent;
 
 		public void OnLuceneDriverChanged (IIndexer    source,
 						   ICollection list_of_added_uris,
@@ -129,5 +133,16 @@ namespace Beagle.Daemon {
 			renamed_uris.Clear ();
 		}
 
+		public void OnLuceneChildIndexableEvent (Indexable[] child_indexables)
+		{
+			if (ChildIndexableEvent != null)
+				ChildIndexableEvent (child_indexables);
+		}
+
+		public void OnLuceneUrisFilteredEvent (FilteredStatus[] list_of_uris_filtered)
+		{
+			if (UrisFilteredEvent != null)
+				UrisFilteredEvent (list_of_uris_filtered);
+		}
 	}
 }
