@@ -75,15 +75,14 @@ namespace WebService_CodeBehind {
 	
 		private WebServiceBackEnd remoteObj = null;
 
-		private searchResult initialQuery(searchRequest req)
+		private SearchResult initialQuery(SearchRequest BeagleQueryRequest)
 		{
-		try {
-			searchResult sr;
+			SearchResult sr;
 			
-			if (req.text == null || req.text.Length == 0 ||
-				(req.text.Length == 1 && req.text[0].Trim()== "")) {
+			if (BeagleQueryRequest.text == null || BeagleQueryRequest.text.Length == 0 ||
+				(BeagleQueryRequest.text.Length == 1 && BeagleQueryRequest.text[0].Trim()== "")) {
 			
-				sr = new searchResult();
+				sr = new SearchResult();
 			    sr.statusCode = WebServiceBackEnd.SC_INVALID_QUERY;
 			    sr.statusMsg = "No search terms specified";
 			    return sr;
@@ -94,33 +93,25 @@ namespace WebService_CodeBehind {
 			if (remoteObj == null)
 				remoteObj = new WebServiceBackEnd();
 
-			if (Application["allowGlobalAccess"] == null)
-				Application["allowGlobalAccess"] =  remoteObj.allowGlobalAccess;
-
 			bool isLocalReq = HttpContext.Current.Request.Url.IsLoopback;
 				
-			if ((remoteObj == null) || !((bool)Application["allowGlobalAccess"] ||
-				isLocalReq) ) 	{
+			if ((remoteObj == null) || !(remoteObj.allowGlobalAccess ||	isLocalReq) ) 	{
 
 				return restrictedAccessResult();
 			}
 
-			sr = remoteObj.doQuery(req, isLocalReq);
+			sr = remoteObj.doQuery(BeagleQueryRequest, isLocalReq);
 			return sr;
-		 }
-		 catch (Exception e) {
-			throw e;
-		 }
-	  }
+	   }
 	
 	[WebMethod(Description = "Full object interface to Beagle")]
 	[System.Web.Services.Protocols.SoapDocumentMethodAttribute(
 	"http://www.gnome.org/projects/beagle/webservices/BeagleQuery",
 	RequestNamespace="http://www.gnome.org/projects/beagle/webservices",
 	ResponseNamespace="http://www.gnome.org/projects/beagle/webservices")]
-	public searchResult BeagleQuery(searchRequest req)
+	public SearchResult BeagleQuery(SearchRequest BeagleQueryRequest)
 	{
-		return initialQuery(req);
+		return initialQuery(BeagleQueryRequest);
 	}
 	
 	[WebMethod(Description = "Simple Interface to Beagle")]
@@ -128,19 +119,19 @@ namespace WebService_CodeBehind {
 	"http://www.gnome.org/projects/beagle/webservices/SimpleQuery",
 	RequestNamespace="http://www.gnome.org/projects/beagle/webservices",
 	ResponseNamespace="http://www.gnome.org/projects/beagle/webservices")]
-	public searchResult SimpleQuery(string text)
+	public SearchResult SimpleQuery(string text)
 	{
-		searchResult sr;
+		SearchResult sr;
 			
 		if (text == null || text.Trim() == "") {
 			
-			sr = new searchResult();
+			sr = new SearchResult();
 			sr.statusCode = WebServiceBackEnd.SC_INVALID_QUERY;
 			sr.statusMsg = "No search terms specified";
 			return sr;
 		}
 			
-		searchRequest srq = new searchRequest();
+		SearchRequest srq = new SearchRequest();
 		
 		srq.text = new string[1];
 		srq.text[0] = text.Trim();
@@ -155,19 +146,19 @@ namespace WebService_CodeBehind {
 	"http://www.gnome.org/projects/beagle/webservices/SimpleQuery2",
 	RequestNamespace="http://www.gnome.org/projects/beagle/webservices",
 	ResponseNamespace="http://www.gnome.org/projects/beagle/webservices")]
-	public searchResult SimpleQuery2(string text, string mimeType, string source, string queryDomain)
+	public SearchResult SimpleQuery2(string text, string mimeType, string source, string queryDomain)
 	{
-		searchResult sr;
+		SearchResult sr;
 			
 		if (text == null || text.Trim() == "") {
 			
-			sr = new searchResult();
+			sr = new SearchResult();
 			sr.statusCode = WebServiceBackEnd.SC_INVALID_QUERY;
 			sr.statusMsg = "No search terms specified";
 			return sr;
 		}
 			
-		searchRequest srq = new searchRequest();
+		SearchRequest srq = new SearchRequest();
 		
 		srq.text = new string[1];
 		srq.text[0] = text.Trim();
@@ -229,13 +220,12 @@ namespace WebService_CodeBehind {
 	"http://www.gnome.org/projects/beagle/webservices/GetMoreResults",
 	RequestNamespace="http://www.gnome.org/projects/beagle/webservices",
 	ResponseNamespace="http://www.gnome.org/projects/beagle/webservices")]
-	public searchResult GetMoreResults(string searchToken, int index)
-	{
-		try {		
-				searchResult sr;
+	public SearchResult GetMoreResults(string searchToken, int index)
+	{		
+			SearchResult sr;
 			
 			if (searchToken == null | searchToken == "")  {
-				sr = new searchResult();
+				sr = new SearchResult();
 				sr.statusCode = WebServiceBackEnd.SC_INVALID_SEARCH_TOKEN;
 				sr.statusMsg = "Invalid Search Token";
 			}
@@ -244,32 +234,58 @@ namespace WebService_CodeBehind {
 			
 			if (remoteObj == null)
 				remoteObj = new WebServiceBackEnd();
-
-			if (Application["allowGlobalAccess"] == null)
-				Application["allowGlobalAccess"] =  remoteObj.allowGlobalAccess;
 			
 			bool isLocalReq = HttpContext.Current.Request.Url.IsLoopback;
 				
-			if ((remoteObj == null) || !((bool)Application["allowGlobalAccess"] ||
-				isLocalReq) ) 	{
-
+			if ((remoteObj == null) || !(remoteObj.allowGlobalAccess ||	isLocalReq) ) 	{
+				
 				return restrictedAccessResult();
 			}
 
 			sr = remoteObj.getMoreResults(searchToken, index, isLocalReq);
 			return sr;
+	}
+		
+	[WebMethod(Description = "Common Interface to get Snippets for results")]
+	[System.Web.Services.Protocols.SoapDocumentMethodAttribute(
+	"http://www.gnome.org/projects/beagle/webservices/GetSnippets",
+	RequestNamespace="http://www.gnome.org/projects/beagle/webservices",
+	ResponseNamespace="http://www.gnome.org/projects/beagle/webservices")]
+	public HitSnippet[] GetSnippets(string searchToken, int[] hitIds)
+	{	
+		HitSnippet[] response;
+					
+		if (searchToken == null | searchToken == "")  {
+			response = new HitSnippet[0];
+			return response;
 		}
-		catch (Exception e)
-		{
-			throw e;
+				
+		remoteChannel.Register();
+			
+		if (remoteObj == null)
+			remoteObj = new WebServiceBackEnd();
+
+		bool isLocalReq = HttpContext.Current.Request.Url.IsLoopback;
+					
+		if ((remoteObj == null) || !(remoteObj.allowGlobalAccess ||	isLocalReq) ) 	{
+				
+			response = new HitSnippet[0];
+			return response;
 		}
+
+		if (hitIds.Length < 1)
+			response = new HitSnippet[0];
+		else
+			response = remoteObj.getSnippets(searchToken, hitIds);
+			
+		return response;
 	}
 
 	private static string localReqOnlyMsg = "Beagle web service unavailable or access restricted to local address only !";
 
-	private searchResult restrictedAccessResult()
+	private SearchResult restrictedAccessResult()
 	{
-		searchResult sr = new searchResult();
+		SearchResult sr = new SearchResult();
 
 		sr.totalResults = 0;
 
